#import "MyOpenGLView.h"
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>

@implementation MyOpenGLView

- (id)initWithFrame:(NSRect)frameRect
{
	NSOpenGLPixelFormatAttribute		attributes[]={NSOpenGLPFADepthSize, 16, nil};

    // This next line calls our parent's init routine
    self = [super initWithFrame:frameRect pixelFormat:[[[NSOpenGLPixelFormat alloc] initWithAttributes:attributes] autorelease]];
    
    [self setup];
	
	// start timer
	worldTime = [NSDate timeIntervalSinceReferenceDate];
	[[NSTimer scheduledTimerWithTimeInterval: 0.0001 target: self selector:@selector(doTimeStep:) userInfo: nil repeats: YES] retain];
    
    return self;
}
	
- (void)setup
{
	[self setupGL];
	[self initScene];
}

- (void)setupGL
{
	// initilize opengl	
    [[self openGLContext] makeCurrentContext];
	
	// set the background color
	glClearColor(0.29, 0.42, 0.58, 0.0);
	
	// set up a single white light
	GLfloat lightColor[] = { 0.7f, 0.7f, 0.7f, 0.0 };
	GLfloat lightAmbientColor[] = { 0.125f, 0.125f, 0.125f, 0.0 };
	GLfloat lightPoition[] = { 500.0f, 200.0f, 500.0f, 0.0 };
	
	glLightfv(GL_LIGHT0, GL_POSITION, lightPoition);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbientColor);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	
	//enable depth testing
    glEnable(GL_DEPTH_TEST);
}

- (void)initScene
{
	// create the newton world
	nWorld = NewtonCreate (NULL, NULL);

	// create the collsion shape
	NewtonCollision *collision = NewtonCreateBox (nWorld, 2.0f, 2.0f, 2.0f, NULL); 

	// create the ridid body
	ribidBodyBox = NewtonCreateBody (nWorld, collision);

	// Get Rid of the collision
	NewtonReleaseCollision (nWorld, collision);

	// set the body mass and inertia
	NewtonBodySetMassMatrix (ribidBodyBox, 1.0f, 1.0f, 1.0f, 1.0f);

	// set the trasformation matrix
	float matrix[16]={1,0,0,0,
					  0,1,0,0,
					  0,0,1,0,
					  0,0,0,1};
	NewtonBodySetMatrix (ribidBodyBox, matrix);

	// animate the body by setting the angular veocity
	float omega[4] = {5.0f, 5.0f, 5.0f, 0.0f};
	NewtonBodySetOmega (ribidBodyBox, omega); 
}

- (void)dealloc
{
	// destroy the Newton world
	NewtonDestroy (nWorld);
}

#pragma mark -

- (void) doTimeStep:(id)sender
{
	[self setNeedsDisplay:YES];
}

- (void)reshape
{
	NSRect bounds = [self bounds];
	
	glViewport(0, 0, (GLsizei) bounds.size.width, (GLsizei) bounds.size.height);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0, (GLfloat) bounds.size.width/(GLfloat) bounds.size.height, 1.0, 30.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -10);
}

- (void)drawRect:(NSRect)r
{
	static float fps = 100;
	
	// updtate the Newton physics world
	NSTimeInterval realTime = [NSDate timeIntervalSinceReferenceDate];
	if([physicsSwitch state] == NSOnState)
		NewtonUpdate (nWorld, realTime-worldTime);
	
	fps += (1/(realTime-worldTime)-fps)/100;
	[fpsTextField performSelectorOnMainThread:@selector(setStringValue:) withObject:[NSString stringWithFormat:@"%0.2f fps", fps] waitUntilDone:NO];
	
	worldTime = realTime;

	// get the matrix from the rigid body
	float matrix[16];
	NewtonBodyGetMatrix(ribidBodyBox, matrix);

	// render the object
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glPushMatrix();
	glMultMatrixf(matrix);
	[self drawBox];
	glPopMatrix();
	
	glFlush();
}

- (void)drawBox
{
	static GLint vertices[] = {-1,-1,-1,  1,-1,-1, 1,1,-1, -1,1,-1, -1,-1,1, 1,-1,1, 1,1,1, -1,1,1};
	static GLubyte allIndices[] = {4, 5, 6, 7,   1, 2, 6, 5,   0, 1, 5, 4,   0, 3, 2, 1,   0, 4, 7, 3,   2, 3, 7, 6};
	
	glEnableClientState (GL_VERTEX_ARRAY);
	glVertexPointer (3, GL_INT, 0, vertices);
	
	glNormal3f(0.0, 0.0, 1.0);
	glDrawElements(GL_QUADS, 4, GL_UNSIGNED_BYTE, allIndices); 
	glNormal3f(1.0, 0.0, 0.0);
	glDrawElements(GL_QUADS, 4, GL_UNSIGNED_BYTE, &allIndices[4]); 
	glNormal3f(0.0, -1.0, 0.0);
	glDrawElements(GL_QUADS, 4, GL_UNSIGNED_BYTE, &allIndices[8]); 
	glNormal3f(0.0, 0.0, -1.0);
	glDrawElements(GL_QUADS, 4, GL_UNSIGNED_BYTE, &allIndices[12]); 
	glNormal3f(-1.0, 0.0, 0.0);
	glDrawElements(GL_QUADS, 4, GL_UNSIGNED_BYTE, &allIndices[16]); 
	glNormal3f(0.0, 1.0, 0.0);
	glDrawElements(GL_QUADS, 4, GL_UNSIGNED_BYTE, &allIndices[20]); 
}

@end
